package net.pijcke.mazes

import javafx.application.Application
import javafx.scene.Scene
import javafx.scene.canvas.Canvas
import javafx.scene.layout.StackPane
import javafx.stage.Stage
import net.pijcke.mazes.generation.BinaryTree
import net.pijcke.mazes.generation.CloseAll
import net.pijcke.mazes.generation.OpenAll
import net.pijcke.mazes.generation.SideWinder
import net.pijcke.mazes.layout.rectangular.*
import net.pijcke.mazes.layout.rectangular.fx.RectangularGridRenderer

class MazesFX : Application() {

    override fun start(primaryStage: Stage) {
        // Create rectangular grid using any implementations, Cell or Graph.
        val grid: RectangularGrid = CellRectangularGrid(42, 42)
//        val grid: RectangularGrid = GraphRectangularGrid(30, 30)

        // Apply some generation algorithm
        SideWinder { _: RectangularGrid -> RectangularGridWalker() }.on(grid)
//        BinaryTree { _: RectangularGrid -> RectangularGridWalker() }.on(grid)
//        OpenAll<CartesianCoordinate, RectangularGrid>().on(grid)
//        CloseAll<CartesianCoordinate, RectangularGrid>().on(grid)

        // We don't use the default renderer so that we can play with parameters.
        val cellSize = 20.0
        val canvas = Canvas(cellSize * grid.width, cellSize * grid.height)
        RectangularGridRenderer(canvas.graphicsContext2D, grid, cellSize, 3.0).render()

        // Simplest scene you can think of.
        primaryStage.scene = Scene(StackPane(canvas))
        primaryStage.show()
    }

}

fun main() {
    Application.launch(MazesFX::class.java)
}
