package net.pijcke.mazes.generation

import net.pijcke.mazes.grid.Coordinate
import net.pijcke.mazes.grid.Grid
import net.pijcke.mazes.grid.GridWalker

class BinaryTree<C: Coordinate<C>, G: Grid<C>>(private val walkerMaker: (G) -> GridWalker<C>): GenerationAlgorithm<C, G> {

    override fun on(grid: G) {
        val walker = walkerMaker(grid)
        for (c in walker.makeSequence(grid)) {
            possibleLinkedPositions(c, walker).filter { grid.isWithinBounds(it) }.randomOrNull()?.let {
                grid.link(c, it)
            }
        }
    }

    private fun possibleLinkedPositions(position: C, walker: GridWalker<C>): List<C> {
        return walker.unitVectors().map { it(position) }
    }

}
