package net.pijcke.mazes.generation

import net.pijcke.mazes.grid.Coordinate
import net.pijcke.mazes.grid.Grid
import net.pijcke.mazes.grid.GridWalker
import java.util.Random

class SideWinder<C: Coordinate<C>, G: Grid<C>>(private val walkerMaker: (G) -> GridWalker<C>) : GenerationAlgorithm<C, G> {

    override fun on(grid: G) {
        val expandablePositions = mutableListOf<C>()

        val walker = walkerMaker(grid)

        walker.makeSequence(grid).forEach { currentPosition ->
            // Connect top row
            if (!grid.isWithinBounds(walker.unitVectors()[0].invoke(currentPosition))) {
                if (grid.isWithinBounds(walker.unitVectors()[1].invoke(currentPosition))) {
                    widenCluster(grid, currentPosition)
                }
                return@forEach
            }

            expandablePositions.add(currentPosition)
            if (grid.isWithinBounds(walker.unitVectors()[1].invoke(currentPosition)) && Random().nextBoolean()) {
                widenCluster(grid, currentPosition)
            }
            else {
                connectCluster(grid, expandablePositions)
                expandablePositions.clear()
            }
        }
    }

    private fun widenCluster(grid: G, coordinate: C) {
        grid.link(coordinate, walkerMaker(grid).unitVectors()[1].invoke(coordinate))
    }

    private fun connectCluster(grid: G, expandableCoordinates: List<C>) {
        val coordinateToExpand = expandableCoordinates.random()
        grid.link(coordinateToExpand, walkerMaker(grid).unitVectors()[0].invoke(coordinateToExpand))
    }

}
