package net.pijcke.mazes.grid

interface GridWalker<C: Coordinate<C>> {

    fun origin(grid: Grid<C>): C

    fun unitVectors(): List<(C) -> C>

    fun makeSequence(grid: Grid<C>): Sequence<C> = makeSequence(grid, origin(grid), unitVectors())

    private fun makeSequence(grid: Grid<C>, startingPoint: C, remainingUnitVectors: List<(C) -> C>): Sequence<C> {
        if (remainingUnitVectors.isEmpty()) {
            return sequenceOf(startingPoint)
        }

        val curF = remainingUnitVectors.first()
        val nextVectors = remainingUnitVectors.drop(1)
        return generateSequence(startingPoint) { curF(it) }
            .takeWhile { grid.isWithinBounds(it) }
            .flatMap { makeSequence(grid, it, nextVectors) }
    }

}
