package net.pijcke.mazes.layout.rectangular.fx

import javafx.scene.canvas.GraphicsContext
import net.pijcke.mazes.layout.rectangular.CartesianCoordinate
import net.pijcke.mazes.layout.rectangular.RectangularGrid
import net.pijcke.mazes.layout.rectangular.RectangularGridWalker
import net.pijcke.mazes.ui.GridRenderer

class RectangularGridRenderer(
    override val context: GraphicsContext,
    override val grid: RectangularGrid,
    private val cellSize: Double,
    private val inset: Double
): GridRenderer<CartesianCoordinate, RectangularGrid> {

    override fun clear() {
        context.clearRect(0.0, 0.0, cellSize * grid.width, cellSize * grid.height)
    }

    override fun render() {
        drawBackground()
        drawCells()
    }

    private fun drawBackground() {
        context.fill = javafx.scene.paint.Color.WHITE
        context.fillRect(0.0, 0.0, 1.0 * cellSize * grid.width, 1.0 * cellSize * grid.height)
    }

    private fun drawCells() {
        context.stroke = javafx.scene.paint.Color.BLACK
        RectangularGridWalker().makeSequence(grid).forEach { drawCell(it) }
    }

    private fun drawCell(position: CartesianCoordinate) {
        val x = 1.0 * position.x * cellSize
        val y = 1.0 * position.y * cellSize

        if (grid.linked(position, position.north())) {
            context.strokeLine(x + inset, y + cellSize - inset, x + inset, y + cellSize)
            context.strokeLine(x + cellSize - inset, y + cellSize - inset, x + cellSize - inset, y + cellSize)
        }
        else {
            context.strokeLine(x + inset, y + cellSize - inset, x + cellSize - inset, y + cellSize - inset)
        }

        if (grid.linked(position, position.south())) {
            context.strokeLine(x + inset, y + inset, x + inset, y)
            context.strokeLine(x + cellSize - inset, y + inset, x + cellSize - inset, y)
        }
        else {
            context.strokeLine(x + inset, y + inset, x + cellSize - inset, y + inset)
        }

        if (grid.linked(position, position.west())) {
            context.strokeLine(x, y + inset, x + inset, y + inset)
            context.strokeLine(x, y + cellSize - inset, x + inset, y + cellSize - inset)
        }
        else {
            context.strokeLine(x + inset, y + inset, x + inset, y + cellSize - inset)
        }

        if (grid.linked(position, position.east())) {
            context.strokeLine(x + cellSize, y + inset, x + cellSize - inset, y + inset)
            context.strokeLine(x + cellSize, y + cellSize - inset, x + cellSize - inset, y + cellSize - inset)
        }
        else {
            context.strokeLine(x + cellSize - inset, y + inset, x + cellSize - inset, y + cellSize - inset)
        }

    }

}
